// main.cpp
#include "global.h"

GLOBALDATA  gl_Data;

// The main program entry point. Differs for Windows and Linux.
#if PLATFORM == MY_PLATFORM_WIN32
// ------------------------------------------------------------------------------------------------
irr::s32 __stdcall WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, irr::s32 nCmdShow)
#else
irr::s32 main(irr::s32 argc, char **argv)
// ------------------------------------------------------------------------------------------------
#endif
{
	if (!gl_Init()) return(0);
	
	// -------------------------------------------------
	while(gl_Data.pIrrlichtDevice->run())
	{
		irr::u32 fOldtime = gl_Data.pIrrlichtDevice->getTimer()->getRealTime(); 
		
		gl_Data.pVideoDriver->beginScene(true, true, irr::video::SColor(0,0,0,0)); // Clear backbuffer, clear zbuffer, color
		gl_Data.pSceneManager->drawAll(); 
		gl_Data.pGUIEnv->drawAll();
		gl_Data.pVideoDriver->endScene();

		gl_Loop();
	}
	// -------------------------------------------------

	gl_Exit();	

	return(0);
}

// ------------------------------------------------------------------------------------------------
bool gl_Init(void)
// ------------------------------------------------------------------------------------------------
{
	srand( (unsigned)time(NULL));

	// -------------------------------------------------
	gl_Data.ScreenXY.Width  = 1024; 
	gl_Data.ScreenXY.Height = 768;
	gl_Data.uColorDepth = 32; // ignored in windowed mode
	
	#if PLATFORM == MY_PLATFORM_WIN32 // -------------------------------------- WIN32

		gl_Data.pIrrlichtDevice = irr::createDevice(
								irr::video::EDT_DIRECT3D9, // EDT_NULL, EDT_SOFTWARE, EDT_DIRECT3D8, EDT_DIRECT3D9, EDT_OPENGL
								gl_Data.ScreenXY, // Screen resolution
								gl_Data.uColorDepth, // bits per pixel (fullscreen only)
								false,  // fullscreen? better be false when debugging
								false,  // create stencil buffer (for e.g. stencil buffer shadows)
								false,  // use vsync
								&gl_Data.Input,   // an IEventReceiver* (user created). 
								IRRLICHT_SDK_VERSION);		

	#else  // -------------------------------------- LINUX
		
		gl_Data.pIrrlichtDevice = irr::createDevice(
								irr::video::EDT_OPENGL, // EDT_NULL, EDT_SOFTWARE, EDT_DIRECTX8, EDT_DIRECTX9, EDT_OPENGL. 
								gl_Data.ScreenXY, // Screen resolution. 
								gl_Data.uColorDepth, // bits per pixel (fullscreen only)
								false,    // fullscreen? better be false when debugging
								false,    // create stencil buffer (for e.g. stencil buffer shadows)
								false,    // use vsync
								&gl_Data.Input,	  // an IEventReceiver* (user created)
								IRRLICHT_SDK_VERSION);
		
	#endif // --------------------------------------
	
	if (!gl_Data.pIrrlichtDevice) return(false);

	gl_Data.pVideoDriver  = gl_Data.pIrrlichtDevice->getVideoDriver();
	if (!gl_Data.pVideoDriver) return(false);

	gl_Data.pSceneManager = gl_Data.pIrrlichtDevice->getSceneManager();
	if (!gl_Data.pSceneManager) return(false);

	gl_Data.pGUIEnv = gl_Data.pIrrlichtDevice->getGUIEnvironment();
	if (!gl_Data.pGUIEnv) return(false);

	gl_Data.pLogger       = gl_Data.pIrrlichtDevice->getLogger();
	if (!gl_Data.pLogger) return(false);

	// -------------------------------------------------
	// Driver presets

	gl_Data.pVideoDriver->setTextureCreationFlag(irr::video::ETCF_ALWAYS_32_BIT, true);

	// -------------------------------------------------
	// Internal objects
	
	if (!gl_Data.Input.Init())	return(0);
	if (!gl_Data.Camera.Init()) return(0);
	if (!gl_Data.Map.Init())	return(0);
	if (!gl_Data.Gui.Init())    return(0);
	if (!gl_Data.Debug.Init())  return(0);
	
	// Debug
	
	DB("gl_Init finished");
    return(true);
}

// ------------------------------------------------------------------------------------------------
void gl_Loop(void)
// ------------------------------------------------------------------------------------------------
{
	gl_Data.Input.Loop();
	gl_Data.Camera.Loop();
	gl_Data.Map.Loop();
	gl_Data.Gui.Loop();
	gl_Data.Debug.Loop();
}

// ------------------------------------------------------------------------------------------------
void gl_Exit(void)
// ------------------------------------------------------------------------------------------------
{
	gl_Data.Input.Exit();
	gl_Data.Camera.Exit();
	gl_Data.Map.Exit();
	gl_Data.Gui.Exit();
	gl_Data.Debug.Exit();
	
	DB("gl_Exit finished");
}

